<?php
/**
 * Zender - Multiple Payment Gateway Plugin
 * @author Titan Systems <mail@titansystems.ph>
 **/

error_reporting(E_ALL);

require "vendor/autoload.php";

require "../mvc_functions.php";

use App\Components\Payment\PaytmResponse;
use App\Components\Payment\PaystackResponse;
use App\Components\Payment\StripeResponse;
use App\Components\Payment\RazorpayResponse;
use App\Components\Payment\InstamojoResponse;
use App\Components\Payment\BitPayResponse;
use App\Components\Payment\MercadopagoResponse;
use App\Components\Payment\PayUmoneyResponse;

get_configs("../../configurations/cc_env.inc", "../../configurations/cc_ver.inc");

define("site_root", "//" . str_replace("/system/plugins/paypage", "", env["siteurl"] . env["port"] . env["subdir"]));

$guzzle = new GuzzleHttp\Client();

$zenderConfig = json_decode($guzzle->get(site_root . "/paypage/config?token=" . env["systoken"], [
    "allow_redirects" => true,
    "http_errors" => false
])->getBody()->getContents(), true);

if($zenderConfig["status"] == 200):
    $configData = $zenderConfig["data"];
    define("config_data", $configData);
else:
    response(500, false);
endif;
 
$requestData = $_REQUEST;

if(!isset($requestData["uhash"]))
    response(500, false);

define("uhash", $requestData["uhash"]);

switch($requestData["paymentOption"]):
    case "paytm":
        $paytmResponse  = new PaytmResponse();

        $paytmData = $paytmResponse->getPaytmPaymentData($requestData);

        if ($paytmData["STATUS"] == "TXN_SUCCESS"):
            $paymentResponseData = [
                "status" => true,
                "rawData" => $paytmData,
                "data" => preparePaymentData($paytmData["ORDERID"], $paytmData["TXNAMOUNT"], $paytmData["TXNID"], "paytm")
            ];
        else:
            $paymentResponseData = [
                "status" => false,
                "rawData" => $paytmData,
                "data" => preparePaymentData($paytmData["ORDERID"], $paytmData["TXNAMOUNT"], $paytmData["TXNID"], "paytm")
            ];
        endif;

        paymentResponse($paymentResponseData);
    case "instamojo":
        if ($requestData["payment_status"] == "Credit"):
            $instamojoResponse  = new InstamojoResponse();

            $instamojoData = $instamojoResponse->getInstamojoPaymentData($requestData);

            $paymentResponseData = [
                "status" => true,
                "rawData" => $instamojoData,
                "data" => preparePaymentData($requestData["orderId"], $instamojoData["amount"], $instamojoData["payment_id"], "instamojo")
            ];
        else:
            $paymentResponseData = [
                "status" => false,
                "rawData" => $requestData,
                "data" => preparePaymentData($requestData["orderId"], $instamojoData["amount"], null, "instamojo")
            ];
        endif;

        paymentResponse($paymentResponseData);
    case "paystack":
        $requestData = json_decode($requestData["response"], true);

        if (isset($requestData["status"]) and $requestData["status"] == "success"):
            $paymentResponseData = [
                "status" => true,
                "rawData" => $requestData,
                "data" => preparePaymentData($requestData["data"]["reference"], $requestData["data"]["amount"], $requestData["data"]["reference"], "paystack")
            ];  
        else:
            $paymentResponseData = [
                "status" => false,
                "rawData" => $requestData,
                "data" => preparePaymentData($requestData["data"]["reference"], $requestData["data"]["amount"], $requestData["data"]["reference"], "paystack")
            ];
        endif;

        paymentResponse($paymentResponseData);
    case "stripe":
        $stripeResponse = new StripeResponse();

        $stripeData = $stripeResponse->retrieveStripePaymentData($requestData["stripe_session_id"]);

        if (isset($stripeData["status"]) and $stripeData["status"] == "succeeded"):
            $paymentResponseData = [
                "status" => true,
                "rawData" => $stripeData,
                "data" => preparePaymentData($requestData["orderId"], $stripeData->amount, $stripeData->charges->data[0]["balance_transaction"], "stripe")
            ];
        else:
            $paymentResponseData = [
                "status" => false,
                "rawData" => $stripeData,
                "data" => preparePaymentData($requestData["orderId"], $stripeData->amount, null, "stripe")
            ];
        endif;

        paymentResponse($paymentResponseData);
    case "razorpay":
        $orderId = $requestData["orderId"];

        $requestData = json_decode($requestData["response"], true);

        if (isset($requestData["status"]) and $requestData["status"] == "captured"):
            $paymentResponseData = [
                "status" => true,
                "rawData" => $requestData,
                "data" => preparePaymentData($orderId, $requestData["amount"], $requestData["id"], "razorpay")
            ];
        else:
            $paymentResponseData = [
                "status" => false,
                "rawData" => $requestData,
                "data" => preparePaymentData($orderId, $requestData["amount"], $requestData["id"], "razorpay")
            ];
        endif;

        paymentResponse($paymentResponseData);
    case "bitpay":
        $paymentResponseData = [
            "status" => true,
            "rawData" => $requestData,
            "data" => preparePaymentData($requestData["orderId"], $requestData["amount"], $requestData["orderId"], "bitpay")
        ];

        paymentResponse($paymentResponseData);
    case "bitpay-ipn":
        $bitpayResponse = new BitPayResponse;
        $rawPostData = file_get_contents("php://input");
        $ipnData = $bitpayResponse->getBitPayPaymentData($rawPostData);
        if ($ipnData["status"] == "success") {
            // code here
        } else {
            // code here
        }

        break;
    case "mercadopago":
        switch($requestData["collection_status"]):
            case "approved":
                $paymentResponseData = [
                    "status" => true,
                    "rawData" => $requestData,
                    "data" => preparePaymentData($requestData["order_id"], $requestData["amount"], $requestData["collection_id"], "mercadopago")
                ];

                break;
            case "pending":
                $paymentResponseData = [
                    "status" => "pending",
                    "rawData" => $requestData,
                    "data" => preparePaymentData($requestData["order_id"], $requestData["amount"], $requestData["collection_id"], "mercadopago")
                ];

                break;
            default:
                $paymentResponseData = [
                    "status" => false,
                    "rawData" => $requestData,
                    "data" => preparePaymentData($requestData["order_id"], $requestData["amount"], $requestData["collection_id"], "mercadopago")
                ];
        endswitch;

        paymentResponse($paymentResponseData);
    case "mercadopago-ipn":
        $mercadopagoResponse = new MercadopagoResponse;
        $mercadopagoIpnData = $mercadopagoResponse->getMercadopagoPaymentData($requestData);  

        break;
    case "payumoney":
        $payUmoneyResponse = new PayUmoneyResponse;
        $payUmoneyResponseData = $payUmoneyResponse->getPayUmoneyPaymentResponse($requestData);

        if ($payUmoneyResponseData["status"] == "success"):
            $paymentResponseData = [
                "status" => true,
                "order_id" => $payUmoneyResponseData["raw_Data"],
                "rawData" => $payUmoneyResponseData["raw_Data"],
                "data" => preparePaymentData($payUmoneyResponseData["order_id"], $payUmoneyResponseData["amount"], $payUmoneyResponseData["txn_id"], "payumoney")
            ];
        else:
            $paymentResponseData = [
                "status" => false,
                "order_id" => "",
                "rawData" => $payUmoneyResponseData["raw_Data"],
                "data" => preparePaymentData($payUmoneyResponseData["order_id"], $payUmoneyResponseData["amount"], $payUmoneyResponseData["txn_id"], "payumoney")
            ];
        endif;

        paymentResponse($paymentResponseData);
    default;
        response(500, false);
endswitch;

function paymentResponse($paymentResponseData)
{
    global $guzzle, $configData, $requestData;

    if ($paymentResponseData["status"] === true) {
        $verify = json_decode($guzzle->post("{$configData["zender_url"]}/paypage/verify", [
            "form_params" => [
                "token" => env["systoken"],
                "hash" => uhash,
                "gateway" => $paymentResponseData["data"]["payment_gateway"]
            ],
            "allow_redirects" => true,
            "http_errors" => false
        ])->getBody()->getContents(), true);

        if($verify["status"] == 200):
            header("location: {$configData["zender_url"]}/paypage/success?hash=" . uhash);
        else:
            header("location: {$configData["zender_url"]}/paypage/failed?hash=" . uhash);
        endif;
    } elseif ($paymentResponseData["status"] === "pending") {
        header("location: {$configData["zender_url"]}/paypage/pending?hash=" . uhash);
    } else {
        header("location: {$configData["zender_url"]}/paypage/failed?hash=" . uhash);
    }

    die;
}

function preparePaymentData($orderId, $amount, $txnId, $paymentGateway)
{
    return [
        "order_id" => $orderId,
        "amount" => $amount,
        "payment_reference_id" => $txnId,
        "payment_gateway" => $paymentGateway
    ];
}
