<?php
/**
 * Zender - Multiple Payment Gateway Plugin
 * @author Titan Systems <mail@titansystems.ph>
 **/

error_reporting(E_ALL);

require "vendor/autoload.php";

require "../mvc_functions.php";

use App\Components\Payment\PaymentProcess;
use App\Service\PaytmService;
use App\Service\InstamojoService;
use App\Service\PaystackService;
use App\Service\RazorpayService;
use App\Service\StripeService;
use App\Service\BitPayService;
use App\Service\MercadopagoService;
use App\Service\PayUmoneyService;

get_configs("../../configurations/cc_env.inc", "../../configurations/cc_ver.inc");

define("site_root", "//" . str_replace("/system/plugins/paypage", "", env["siteurl"] . env["port"] . env["subdir"]));

$guzzle = new GuzzleHttp\Client();

$zenderConfig = json_decode($guzzle->get(site_root . "/paypage/config?token=" . env["systoken"], [
    "allow_redirects" => true,
    "http_errors" => false
])->getBody()->getContents(), true);

if($zenderConfig["status"] == 200):
    $configData = $zenderConfig["data"];
    define("config_data", $configData);
else:
    response(500, false);
endif;

$paytmService = new PaytmService();
$instamojoService = new InstamojoService();
$paystackService = new PaystackService();
$razorpayService = new RazorpayService();
$stripeService = new StripeService();
$bitPayService = new BitPayService();
$mercadopagoService = new MercadopagoService();
$payUmoneyService = new PayUmoneyService();
$paymentProcess = new PaymentProcess(
    $paytmService,
    $instamojoService,
    $paystackService,
    $razorpayService,
    $stripeService,
    $bitPayService,
    $mercadopagoService,
    $payUmoneyService,
);

$gump = new GUMP();

if(isset($_POST) && !empty($_POST)):
    $insertData = $gump->sanitize($_POST);

    $validation = GUMP::is_valid($insertData, array(
        "paymentOption" => "required"
    ));

    $paymentOption = $insertData["paymentOption"];

    if ($validation === true):
        $paymentData = $paymentProcess->getPaymentData($insertData);

        $paymentData["paymentOption"] = $paymentOption;

        switch($paymentOption):
            case "paytm":
                $paymentData["merchantForm"] = getPaytmMerchantForm($paymentData);
                die(json_encode($paymentData));
            case "razorpay":
                die(json_encode(array_values($paymentData)[0]));
            default:
                die(json_encode($paymentData));
        endswitch;
    else:
        $validationMessage = [];

        foreach ($validation as $valid) {
            $validationMessage["validationMessage"][] = strip_tags($valid);
        }

        echo json_encode($validationMessage);
    endif;
else:
    response(500, false);
endif;
